@@ -1112,7 +1112,9 @@ pub mod testing {
 
 #[test]
 fn component_definition_properties() {
+    sixtyfps_rendering_backend_testing::init();
     let mut compiler = ComponentCompiler::default();
+    compiler.set_style("fluent".into());
     let comp_def = spin_on::spin_on(
         compiler.build_from_source(
             r#"
@@ -1158,7 +1160,9 @@ fn component_definition_properties() {
 
 #[test]
 fn component_definition_properties2() {
+    sixtyfps_rendering_backend_testing::init();
     let mut compiler = ComponentCompiler::default();
+    compiler.set_style("fluent".into());
     let comp_def = spin_on::spin_on(
         compiler.build_from_source(
             r#"
@@ -1186,7 +1190,9 @@ fn component_definition_properties2() {
 
 #[test]
 fn globals() {
+    sixtyfps_rendering_backend_testing::init();
     let mut compiler = ComponentCompiler::default();
+    compiler.set_style("fluent".into());
     let definition = spin_on::spin_on(
         compiler.build_from_source(
             r#"
@@ -1302,7 +1308,9 @@ fn globals() {
 
 #[test]
 fn component_definition_struct_properties() {
+    sixtyfps_rendering_backend_testing::init();
     let mut compiler = ComponentCompiler::default();
+    compiler.set_style("fluent".into());
     let comp_def = spin_on::spin_on(
         compiler.build_from_source(
             r#"
