@@ -17,13 +17,12 @@ use corelib::rtti::AnimatedBindingKind;
 use corelib::window::{WindowHandleAccess, WindowRc};
 use corelib::{Brush, Color, PathData, SharedString, SharedVector};
 use sixtyfps_compilerlib::expression_tree::{
-    BindingExpression, BuiltinFunction, EasingCurve, Expression, Path as ExprPath,
-    PathElement as ExprPathElement,
+    BuiltinFunction, EasingCurve, Expression, Path as ExprPath, PathElement as ExprPathElement,
 };
 use sixtyfps_compilerlib::langtype::Type;
 use sixtyfps_compilerlib::object_tree::ElementRc;
 use sixtyfps_corelib as corelib;
-use std::collections::{BTreeMap, HashMap};
+use std::collections::HashMap;
 use std::rc::Rc;
 
 pub trait ErasedPropertyInfo {
@@ -305,7 +304,20 @@ pub fn eval_expression(expression: &Expression, local_context: &mut EvalLocalCon
                     let popup = popup_list.iter().find(|p| Rc::ptr_eq(&p.component, &pop_comp)).unwrap();
                     let x = load_property_helper(local_context.component_instance, &popup.x.element(), popup.x.name()).unwrap();
                     let y = load_property_helper(local_context.component_instance, &popup.y.element(), popup.y.name()).unwrap();
-                    crate::dynamic_component::show_popup(popup, x.try_into().unwrap(), y.try_into().unwrap(), component.borrow(), window_ref(component).unwrap());
+
+                    generativity::make_guard!(guard);
+                    let enclosing_component =
+                        enclosing_component_for_element(&popup.parent_element, component, guard);
+                    let parent_item_info = &enclosing_component.component_type.items[popup.parent_element.borrow().id.as_str()];
+                    let parent_item_comp = enclosing_component.self_weak().get().unwrap().upgrade().unwrap();
+                    let parent_item = corelib::items::ItemRc::new(vtable::VRc::into_dyn(parent_item_comp), parent_item_info.item_index());
+
+                    crate::dynamic_component::show_popup(
+                        popup,
+                        sixtyfps_corelib::graphics::Point::new(x.try_into().unwrap(), y.try_into().unwrap()),
+                        component.borrow(),
+                        window_ref(component).unwrap(),
+                        &parent_item);
                     Value::Void
                 } else {
                     panic!("internal error: argument to SetFocusItem must be an element")
@@ -418,7 +430,7 @@ pub fn eval_expression(expression: &Expression, local_context: &mut EvalLocalCon
                     let item_ref = unsafe { item_info.item_from_component(enclosing_component.as_ptr()) };
 
                     let window = window_ref(component).unwrap();
-                    item_ref.as_ref().layouting_info(crate::eval_layout::to_runtime(*orient), window).into()
+                    item_ref.as_ref().layout_info(crate::eval_layout::to_runtime(*orient), window).into()
                 } else {
                     panic!("internal error: incorrect arguments to ImplicitLayoutInfo {:?}", arguments);
                 }
@@ -519,6 +531,9 @@ pub fn eval_expression(expression: &Expression, local_context: &mut EvalLocalCon
 
                     }
                 }
+                sixtyfps_compilerlib::expression_tree::ImageReference::EmbeddedTexture { .. } => {
+                    todo!()
+                }
             }.unwrap_or_else(|_| {
                 eprintln!("Could not load image {:?}",resource_ref );
                 Default::default()
@@ -746,10 +761,13 @@ pub fn store_property(
 ) -> Result<(), SetPropertyError> {
     generativity::make_guard!(guard);
     let enclosing_component = enclosing_component_for_element(element, component_instance, guard);
-    let maybe_animation = crate::dynamic_component::animation_for_property(
-        enclosing_component,
-        element.borrow().bindings.get(name).map_or(&None, |b| &b.animation),
-    );
+    let maybe_animation = match element.borrow().bindings.get(name) {
+        Some(b) => crate::dynamic_component::animation_for_property(
+            enclosing_component,
+            &b.borrow().animation,
+        ),
+        None => crate::dynamic_component::animation_for_property(enclosing_component, &None),
+    };
 
     let component = element.borrow().enclosing_component.upgrade().unwrap();
     if element.borrow().id == component.root_element.borrow().id {
@@ -980,13 +998,13 @@ pub(crate) fn enclosing_component_instance_for_element<'a, 'old_id, 'new_id>(
 }
 
 pub fn new_struct_with_bindings<ElementType: 'static + Default + corelib::rtti::BuiltinItem>(
-    bindings: &BTreeMap<String, BindingExpression>,
+    bindings: &sixtyfps_compilerlib::object_tree::BindingsMap,
     local_context: &mut EvalLocalContext,
 ) -> ElementType {
     let mut element = ElementType::default();
     for (prop, info) in ElementType::fields::<Value>().into_iter() {
         if let Some(binding) = &bindings.get(prop) {
-            let value = eval_expression(binding, local_context);
+            let value = eval_expression(&binding.borrow(), local_context);
             info.set_field(&mut element, value).unwrap();
         }
     }
