@@ -9,7 +9,7 @@
 LICENSE END */
 
 use crate::api::Value;
-use sixtyfps_corelib::model::Model;
+use sixtyfps_corelib::model::{Model, ModelTracker};
 use std::cell::RefCell;
 
 pub struct ValueModel {
@@ -23,6 +23,19 @@ impl ValueModel {
     }
 }
 
+impl ModelTracker for ValueModel {
+    fn attach_peer(&self, peer: sixtyfps_corelib::model::ModelPeer) {
+        if let Value::Model(ref model_ptr) = *self.value.borrow() {
+            model_ptr.model_tracker().attach_peer(peer.clone())
+        }
+        self.notify.attach_peer(peer)
+    }
+
+    fn track_row_count_changes(&self) {
+        self.notify.track_row_count_changes()
+    }
+}
+
 impl Model for ValueModel {
     type Data = Value;
 
@@ -53,11 +66,8 @@ impl Model for ValueModel {
         }
     }
 
-    fn attach_peer(&self, peer: sixtyfps_corelib::model::ModelPeer) {
-        if let Value::Model(ref model_ptr) = *self.value.borrow() {
-            model_ptr.attach_peer(peer.clone())
-        }
-        self.notify.attach(peer)
+    fn model_tracker(&self) -> &dyn ModelTracker {
+        self
     }
 
     fn set_row_data(&self, row: usize, data: Self::Data) {
